import 'package:json_annotation/json_annotation.dart';
import 'package:meta/meta.dart';
import '../message.dart';
import '../preview_data.dart' show PreviewData;
import '../user.dart' show User;
import 'partial_file.dart';
import 'package:objectbox/objectbox.dart';

part 'file_message.g.dart';

/// A class that represents file message.
@Entity()
@Sync()
@JsonSerializable(explicitToJson: true)
class FileMessage extends Message {
  /// Creates a file message.
  @Id(assignable: true)
  int id;

  final author = ToOne<User>();
  FileMessage({
    int? createdAt,
    Map<String, dynamic>? metadata,
    this.mimeType,
    required this.name,
    int? remoteId,
    int? roomId,
    required this.size,
    Status? status,
    MessageType? type,
    int? updatedAt,
    required this.uri,
    this.id = 0,
  }) : super(
          createdAt,
          metadata,
          remoteId,
          roomId,
          status,
          type ?? MessageType.file,
          updatedAt,
        );

  /// Creates a full file message from a partial one.
  FileMessage.fromPartial(
      {int? createdAt,
      required PartialFile partialFile,
      int? remoteId,
      int? roomId,
      Status? status,
      int? updatedAt,
      this.id = 0})
      : mimeType = partialFile.mimeType,
        name = partialFile.name,
        size = partialFile.size,
        uri = partialFile.uri,
        super(
          createdAt,
          partialFile.metadata,
          remoteId,
          roomId,
          status,
          MessageType.file,
          updatedAt,
        );

  /// Creates a file message from a map (decoded JSON).
  factory FileMessage.fromJson(Map<String, dynamic> json) =>
      _$FileMessageFromJson(json);

  /// Converts a file message to the map representation, encodable to JSON.
  @override
  Map<String, dynamic> toJson() => _$FileMessageToJson(this);

  /// Creates a copy of the file message with an updated data.
  /// [metadata] with null value will nullify existing metadata, otherwise
  /// both metadatas will be merged into one Map, where keys from a passed
  /// metadata will overwite keys from the previous one.
  /// [previewData] is ignored for this message type.
  /// [remoteId] and [updatedAt] with null values will nullify existing value.
  /// [status] and [uri] with null values will be overwritten by previous values.
  /// [text] is ignored for this message type.
  @override
  Message copyWith({
    Map<String, dynamic>? metadata,
    PreviewData? previewData,
    int? remoteId,
    Status? status,
    String? text,
    int? updatedAt,
    String? uri,
  }) {
    return FileMessage(
      createdAt: createdAt,
      id: id,
      metadata: metadata == null
          ? null
          : {
              ...this.metadata ?? {},
              ...metadata,
            },
      mimeType: mimeType,
      name: name,
      remoteId: remoteId,
      roomId: roomId,
      size: size,
      status: status ?? this.status,
      updatedAt: updatedAt,
      uri: uri ?? this.uri,
    );
  }

  /// Equatable props
  @override
  List<Object?> get props => [
        author,
        createdAt,
        id,
        metadata,
        mimeType,
        name,
        remoteId,
        roomId,
        size,
        status,
        updatedAt,
        uri,
      ];

  /// Media type
  String? mimeType;

  /// The name of the file
  String name;

  /// Size of the file in bytes
  int size;

  /// The file source (either a remote URL or a local resource)
  String uri;
}
